//+------------------------------------------------------------------+
//|                                               RSI_BB_TIMER_2.mq4 |
//|                                                                  |
//|                                             https://www.mql5.com |
//+------------------------------------------------------------------+

// This version of the bot operates with a timer set to run every few 
// miliseconds and check wether a bar has closed or not

#property copyright "Loite27"
#property link      "https://www.mql5.com"
#property version   "1.00"
#property strict

//----------------------STRATEGY VARIABLES--------------------------------------//
// RSI variables
extern int rsiSessions = 14; // RSI Sessions
extern double rsiLowTradingLevel = 30; // RSI Low Trading Level
extern double rsiLowEnableLevel = 0; // RSI Low Enable Level
extern double rsiUpTradingLevel = 70; // RSI Up Trading Level
extern double rsiUpEnableLevel = 100; // RSI Up Enable Level
bool rsiEnable = True;

// Bollinger Bands variables
extern int bbSessions = 20; // BB Sessions
extern int bbShift = 0;  // BB Shift
extern double bbDeviation = 2; // BB Deviation
//-------------------------------------------------------------------------------//


//-------------------COMMON VARIABLES--------------------------------------------//
// Money management variables
extern double investmentPercentage = 0.05; // Investment percentage. Default = 5%
double investmentAmount = investmentPercentage * AccountEquity();
double maxTradingAmount = 250; // Maximum possible investment defined by broker
double minLot = MarketInfo(Symbol(),MODE_MINLOT); // Minimum investment lot

int barsUntilOperation = MathMax(bbSessions,rsiSessions);

//Time Variables
extern int startHour = 0;
extern int startMinute = 0;
extern int finishHour = 24;
extern int finishMinute = 0;
datetime localServerOffset = 0;
datetime currentBar = 0;
datetime nextBar = 0;
extern int timerMiliseconds = 1;
extern int sleepMilliseconds = 65; // Aprox ping time



// Constants and variables
bool firstTick = True;
const int ArrayResizeFactor = 50;
int OperationsArray [20];
int operationsCount = 0;
int timeExpirySeconds = PeriodSeconds(PERIOD_CURRENT);
int timeFrame = PeriodSeconds(PERIOD_CURRENT);
//-------------------------------------------------------------------------------//


//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit()
  {
   Alert("RSI_BB_TIMER_2 has started in Symbol: ",Symbol()); 
   DrawVerticaltLine();   
   return(INIT_SUCCEEDED);
  }
  
  
//+------------------------------------------------------------------+
//| Expert deinitialization function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
//--- destroy timer
   EventKillTimer();
   Alert("RSI_BB_TIMER_2 has been ended in Symbol: ",Symbol()); 
   DrawVerticaltLine();
      
  }
  
  
//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
void OnTick()
  {    
      localServerOffset = TimeLocal() - TimeCurrent();  
      if (firstTick == True)
      {  
         currentBar = Time[0];       
         nextBar = currentBar + timeFrame; 
         if (TimeDayOfWeek(nextBar)>=5 && TimeHour(nextBar)>=23 && TimeMinute(nextBar)>=55)
         {
            // Dont set the timer if market has ended on friday
         }
         else
         {
            EventSetMillisecondTimer(timerMiliseconds);
            firstTick = False;
         }
         
      
      }
   
  }
  
  
//+------------------------------------------------------------------+
//| Timer function                                                   |
//+------------------------------------------------------------------+
void OnTimer()
  {
//---
      if((TimeLocal()-localServerOffset)>=nextBar)
      {
         EventKillTimer();
         Sleep(sleepMilliseconds);
         CallStrategy(currentBar);
         currentBar = nextBar;
         nextBar = currentBar + timeFrame;
         if (TimeDayOfWeek(nextBar)>=6 && TimeHour(nextBar)>=0 && TimeMinute(nextBar)>=55)
         {
            firstTick = True; // No timer until first tick on monday
         }
         else
         {
            EventSetMillisecondTimer(timerMiliseconds);
         }
      }   
  }
//+------------------------------------------------------------------+


//----------------------COMMON FUNCTIONS------------------------------//

void DrawVerticaltLine()
{
   ObjectCreate("StartLine",OBJ_VLINE,0,Time[0],0);      
   ChartRedraw();
} 

//----------------------STRATEGY FUNCTIONS---------------------------//

bool CheckUserEnableHours(datetime BarTime)
{
   bool hourEnable = True;
   int HH = TimeHour(BarTime);
   int MM = TimeMinute(BarTime);
   
   if(startHour<HH && finishHour>HH)
      {
         hourEnable = True;
      }
      else
      {
         if(startHour==HH)
         {
            if(startMinute>=MM)
            {
               hourEnable = True;
            }
         }
         else
         {
            if(finishHour==HH)
            {
               if(finishMinute<=MM)
               {
                  hourEnable = True;
               }
            }
         }
      }
   return hourEnable;
}

bool CheckBrokerEnable(datetime BarTime)
{
   // Check for broker times of operation (SERVER HOURS: GMT +3)
  
   bool brokerEnable = True;
   /*
   if(TimeDayOfWeek(BarTime) == 6 || TimeDayOfWeek(BarTime) == 0) // if saturday or sunday
   {
      brokerEnable = False;
   }
   else
   {
      if(TimeDayOfWeek(BarTime)==5) // if friday
      {
         if(TimeHour(BarTime)>=3 && TimeHour(BarTime)<22)
         {
            brokerEnable = True;
         }
         else
         {
            brokerEnable = False;
         }
      }
      else
      {
         if(TimeHour(BarTime)>=3 && TimeHour(BarTime)<24)
         {
            brokerEnable = True;
         }
         else
         {
            brokerEnable = False;
         }
      }
   }*/
   return brokerEnable;   
}

bool AmountOfBarsOK(int barsToWait)
{
   if (Bars >= barsToWait)
   {
      return True;
   }
   else
   {
      return False;
   }
}



void CallStrategy(datetime BarTime)
{
   double RSI = iRSI(NULL,PERIOD_CURRENT,rsiSessions,PRICE_CLOSE,iBarShift(Symbol(),timeFrame,BarTime,false));  
   double UpBB = iBands(NULL,PERIOD_CURRENT,bbSessions,bbDeviation,bbShift,PRICE_CLOSE,MODE_UPPER,iBarShift(Symbol(),timeFrame,BarTime,false)); 
   double LowBB = iBands(NULL,PERIOD_CURRENT,bbSessions,bbDeviation,bbShift,PRICE_CLOSE,MODE_LOWER,iBarShift(Symbol(),timeFrame,BarTime,false));
   
   if(CheckUserEnableHours(BarTime) && AmountOfBarsOK(barsUntilOperation) && CheckBrokerEnable(BarTime))
   {  
      if (operationsCount == ArraySize(OperationsArray)-1)
      {
         ArrayResize(OperationsArray,ArraySize(OperationsArray)+ ArrayResizeFactor,0);
      }  
        
      investmentAmount = NormalizeDouble(investmentPercentage * AccountEquity(),1);
      if (investmentAmount > maxTradingAmount)
      {
         investmentAmount = maxTradingAmount;
      }
      else
      {
         if (investmentAmount < minLot)
         {
            investmentAmount = minLot;
         } 
      }
      if(Close[iBarShift(Symbol(),timeFrame,BarTime,false)]>UpBB && RSI>rsiUpTradingLevel && rsiEnable && Close[iBarShift(Symbol(),timeFrame,BarTime,false)]>Open[iBarShift(Symbol(),timeFrame,BarTime,false)])
      {                    
         // Change magic number to identify every transaction
         RefreshRates();
         OperationsArray[operationsCount] = OrderSend(Symbol(),OP_SELL, investmentAmount, Bid, 0, 0, 0, "BO exp:"+IntegerToString(timeExpirySeconds), 1, 0, clrRed);
         operationsCount++;
         rsiEnable = False;
         Alert("Put option attempted on symbol: ",Symbol()," - Error: ",GetLastError());     
      }
      else
      {
         if(Close[iBarShift(Symbol(),timeFrame,BarTime,false)]<LowBB && RSI<rsiLowTradingLevel && rsiEnable && Close[iBarShift(Symbol(),timeFrame,BarTime,false)]<Open[iBarShift(Symbol(),timeFrame,BarTime,false)])
         {     
            // Change magic number to identify every transaction
            RefreshRates();
            OperationsArray[operationsCount] = OrderSend(Symbol(),OP_BUY, investmentAmount, Ask, 0, 0, 0, "BO exp:"+IntegerToString(timeExpirySeconds), 1, 0, clrGreen);
            operationsCount++;
            rsiEnable = False;
            Alert("Call option attempted on symbol: ",Symbol()," - Error: ",GetLastError());
         }
      }            
      
    }
    
    if (RSI > rsiLowEnableLevel && RSI < rsiUpEnableLevel)
    {
      rsiEnable = True;
    }   
}